#!/usr/bin/bash
# Copyright Atomicorp, 2024
# All Rights Reserved

###############
# Globals
###############
VERSION="2.4.2"
MIN_VER="ClamAV 0.103.1"
DEBUG=0

if [ ! -f /var/ossec/etc/client.keys ]; then
        echo "Error: Agent not registered"
        exit 1
fi


ID=$(cat /var/ossec/etc/client.keys | awk '{print $1}')



###########################
# Logging
###########################
LOG_FILE="/var/ossec/logs/atomicorp-api.log"


###########################
#functions
###########################
function log_event () {
    MSG=$1
    echo "`date "+%b %d %H:%M:%S"` ${HOSTNAME} atomicorp-api-module-clamav: ${MSG}" >> ${LOG_FILE}
	logger -t atomicorp-api-module-clamav "${MSG} Agent ID: ${ID} "
}

function dist_detection() {
	if [[ $DEBUG -ge 1 ]]; then
		log_event "DEBUG1: $0: dist_detection START"
	fi
	if [ -f /etc/redhat-release ]; then
		RELEASE=/etc/redhat-release
	elif [ -f /etc/os-release ]; then
		RELEASE=/etc/os-release
	elif [ -f /etc/openvz-release ]; then
		RELEASE=/etc/openvz-release
	elif [ -f /etc/SuSE-release ]; then
		RELEASE=/etc/SuSE-release
	elif [ -f /etc/os-release ]; then
		RELEASE=/etc/os-release
	elif [ -f /etc/lsb-release ]; then
		RELEASE=/etc/lsb-release
	elif [ -f /etc/debian_version ]; then
		RELEASE=/etc/debian_version
	elif [ -f /etc/openvz-release ]; then
		RELEASE=/etc/openvz-release
	elif [ -f /etc/virtuozzo-release ]; then
		RELEASE=/etc/virtuozzo-release
	  	RELEASE=
	elif [[ $OSTYPE == "aix"* ]]; then
	  	PKG=aix
	fi

	PKG=rpm
	if [[ $OSTYPE == "aix"* ]]; then
	  	PKG=aix
	elif egrep -q "(release 5)" $RELEASE ; then
	  	DIST="el5"
	elif egrep -q "(release 6|release 2012)" $RELEASE ; then
	  	DIST="el6"
	elif egrep -q "(release 7|release 2014)" $RELEASE ; then
	  	DIST="el7"
	elif egrep -q "(release 8)" $RELEASE ; then
	  	DIST="el8"
	elif egrep -q "(release 9)" $RELEASE ; then
	  	DIST="el9"
	elif egrep -q "Red Hat Enterprise Linux.* 7" $RELEASE ; then
	  	DIST="el7"
	elif egrep -q "Red Hat Enterprise Linux.* 8" $RELEASE ; then
	  	DIST="el8"
	elif egrep -q "(Amazon Linux 2)" $RELEASE; then
	  	DIST="amazon"
	elif egrep -q "(Amazon Linux AMI|Amazon)" $RELEASE ; then
	  	DIST="el6"
	elif egrep -q "wheezy" $RELEASE ; then
	  	DIST="debian"
	  	PKG=deb
	elif egrep -q "jessie" $RELEASE ; then
	  	DIST="debian"
	  	PKG=deb
	elif egrep -q "stretch" $RELEASE ; then
	  	DIST="debian"
	  	PKG=deb
	elif egrep -q "buster" $RELEASE ; then
	  	DIST="debian"
	  	PKG=deb
	elif egrep -q "bookworm" $RELEASE ; then
	  	DIST="debian"
	  	PKG=deb
	elif egrep -q "lucid" $RELEASE ; then
	  	DIST="debian"
	  	PKG=deb
	elif egrep -q "precise" $RELEASE ; then
	  	DIST="debian"
	  	PKG=deb
	elif egrep -q "Raring Ringtail" $RELEASE ; then
	  	DIST="debian"
	  	PKG=deb
	elif egrep -q "Trusty Tahr" $RELEASE ; then
	  	DIST="ubuntu"
	  	PKG=deb
	elif egrep -q "Xenial" $RELEASE ; then
	  	DIST="ubuntu"
	  	PKG=deb
	elif egrep -q "Bionic" $RELEASE ; then
		DIST="ubuntu"
		PKG=deb
	elif egrep -q "Focal" $RELEASE ; then
		DIST="ubuntu"
		PKG=deb
	elif egrep -q "Jammy" $RELEASE ; then
		DIST="ubuntu"
		PKG=deb
	fi
	if [[ $DEBUG -ge 1 ]]; then
		log_event "DEBUG1: $0: dist_detection DIST: ($DIST) PKG: ($PKG) END"
	fi

}

# Function to check if running in Docker environment
function is_docker() {
	if [ -f "/.dockerenv" ]; then
		return 0  # Running in Docker
	else
		return 1  # Not running in Docker
	fi
}

function clamd_settings() {
	if [[ "$PKG" == "rpm" ]]; then
		FRESHCLAM_CONF="/etc/freshclam.conf"

		if [ -f /etc/clamd.d/scan.conf ]; then
			CLAM_CONF="/etc/clamd.d/scan.conf"
			SCAN_D=true
			if ! grep -q "^LocalSocket" $CLAM_CONF; then
           		sed -i '/^#LocalSocket/s/^#//g' $CLAM_CONF
        	fi
			if id clamupdate &>/dev/null; then
				sed -i "s/^DatabaseOwner.*/DatabaseOwner clamupdate/g" $FRESHCLAM_CONF
				sed -i "s/^User.*/User clamscan/g" $CLAM_CONF
				chown clamupdate:clamupdate -R /var/lib/clamav >/dev/null 2>&1
			fi
		elif [ -f /etc/clamd.conf ]; then
			CLAM_CONF="/etc/clamd.conf"
			if id clamav &>/dev/null; then
				sed -i "s/^DatabaseOwner.*/DatabaseOwner clamav/g" /etc/freshclam.conf
				sed -i "s/^User.*/User clamav/g" /etc/clamd.conf
			fi
		fi

		


	elif [[ "$PKG" == "deb" ]]; then
		CLAM_CONF="/etc/clamav/clamd.conf"
		FRESHCLAM_CONF="/etc/clamav/freshclam.conf"
	fi

	if [[ $DEBUG -ge 1 ]]; then
		log_event "DEBUG1: $0: clamd_settings CLAM_CONF is ($CLAM_CONF)"
	fi

	if ! is_docker ; then
	    systemctl daemon-reload
	    if  [[ $SCAN_D == "true" ]]; then
		systemctl enable clamd@scan
	    else
	    	systemctl enable clamav-daemon
	    fi
	fi


}

clamd_restart() {
	if [[ $DEBUG -ge 1 ]]; then
		log_event "DEBUG1: clamd_restart:: START"
		log_event "DEBUG1: clamd_restart:: ENABLE is $ENABLE"
	fi

	# if ENABLE is 0, then disable and stop clamav-daemon and clamonacc
	if [[ $ENABLE -eq 0 ]]; then
		log_event "DEBUG1: clamd_restart:: ENABLE is $ENABLE but we evaluated this as 0 "

		if ! is_docker ; then
			if  [[ $SCAN_D == "true" ]]; then
				systemctl disable clamd@scan
				systemctl stop clamd@scan
			else
				systemctl disable clamav-daemon
				systemctl stop clamav-daemon
			fi
			if [ -f /usr/lib/systemd/system/clamav-clamonacc.service ]; then
				systemctl disable clamav-clamonacc >/dev/null 2>&1
				systemctl stop clamav-clamonacc >/dev/null 2>&1
			else
				systemctl disable clamonacc >/dev/null 2>&1
				systemctl stop clamonacc >/dev/null 2>&1
			fi
			killall -9 clamonacc >/dev/null 2>&1

		fi
		return 0
	fi

	if ! is_docker ; then
		if  [[ $SCAN_D == "true" ]]; then
			systemctl restart clamd@scan
		else
			systemctl restart clamav-daemon
		fi
		if [[ $DEBUG -ge 1 ]]; then
			log_event "DEBUG1: clamd_restart:: systemctl restart clamav-daemon ($?)"
		fi
		
		if [[ $DEBUG -ge 1 ]]; then
			log_event "DEBUG1: clamd_restart:: INCLUDE_PATHS is ($INCLUDE_PATHS)"
		fi
  	        if [[ $INCLUDE_PATHS -ge 1 ]]; then
        		systemctl enable clamonacc >/dev/null 2>&1
        		systemctl start clamonacc >/dev/null 2>&1
        		if [[ $DEBUG -ge 1 ]]; then
        			log_event "DEBUG1: clamd_restart:: systemctl start clamonacc ($?)"
        		fi
    		else
        		systemctl disable clamonacc >/dev/null 2>&1
        		systemctl stop clamonacc >/dev/null 2>&1
				killall -9 clamonacc >/dev/null 2>&1
        		if [[ $DEBUG -ge 1 ]]; then
           			log_event "DEBUG1: clamd_restart:: systemctl stop clamonacc ($?)"
        		fi
    		fi
	fi
	if [[ $DEBUG -ge 1 ]]; then
		log_event "DEBUG1: $0: clamd_restart:: END"
	fi
}

clamd_version() {
	if [[ $DEBUG -ge 1 ]]; then
		log_event "DEBUG1: $0: clamd_version START"
	fi
	# standard
	if  [ -f /usr/sbin/clamd ]; then
		CLAMD_DAEMON=/usr/sbin/clamd
	elif [ -f /opt/freeware/bin/clamd ]; then
		CLAMD_DAEMON=/opt/freeware/bin/clamd
	else
		echo "Error: clamd not detected"
		exit 1
	fi

	C_VERSION=$(${CLAMD_DAEMON} --version |awk -F/ '{print $1}')
	if [[ "$C_VERSION" < "$MIN_VER" ]]; then
		echo "ERROR: Minimum version $MIN_VER required. $C_VERSION detected"
		echo 
		exit 1
	fi
	if [[ $DEBUG -ge 1 ]]; then
		log_event "DEBUG1: $0: clamd_version ($C_VERSION) END"
	fi

}

freshclam_settings(){
	if [[ $DEBUG -ge 1 ]]; then
		log_event "DEBUG1: freshclam settings updated: START"
	fi

	# if ENABLE=0 then return, we dont want to update freshclam settings
	if [[ $ENABLE -eq 0 ]]; then
		return 0
	fi

	if [[ ${FRESHCLAM_FEED_TYPE} == "official" ]]; then
		sed -i '/^DatabaseCustomURL/d' $FRESHCLAM_CONF
		sed -i '/^ExcludeDatabase/d' $FRESHCLAM_CONF
	else
		if ! grep -q "^DatabaseMirror database.clamav.net" $FRESHCLAM_CONF; then
			echo "DatabaseMirror database.clamav.net" >> $FRESHCLAM_CONF
		fi
		if ! grep -q "^ExcludeDatabase main" $FRESHCLAM_CONF; then
			echo "ExcludeDatabase main" >> $FRESHCLAM_CONF
		fi
		if ! grep -q "^ExcludeDatabase daily" $FRESHCLAM_CONF; then
			echo "ExcludeDatabase daily" >> $FRESHCLAM_CONF
		fi
		if ! grep -q "^ExcludeDatabase daily" $FRESHCLAM_CONF; then
			echo "ExcludeDatabase bytecode" >> $FRESHCLAM_CONF
		fi

		# clear all the DatabaseCustomURL lines that contain atomicorp.com in them
      		sed -i '/DatabaseCustomURL.*atomicorp\.com/d' $FRESHCLAM_CONF

		if [[ $DEBUG -ge 1 ]]; then
			log_event "DEBUG1: freshclam_settings FRESHCLAM_LINUX_SIGS = ${FRESHCLAM_LINUX_SIGS}"
		fi
		# Populate URLs for the DatabaseCustomURL lines
		EXTENSIONS="fp hdb hdu hsb hsu idb ign2 ldb ldu ndb ndu sfp"
		if [[ ${FRESHCLAM_LINUX_SIGS} == "yes" ]]; then
			for ext in $EXTENSIONS; do
				files_to_add+=(
					"Atomicorp-Linux.${ext}"
				)
			done
		else
			rm -f /var/lib/clamav/Atomicorp-Linux.* >/dev/null 2>&1
		fi
		if [[ ${FRESHCLAM_WINDOWS_SIGS} == "yes" ]]; then
			for ext in $EXTENSIONS; do
				files_to_add+=(
					"Atomicorp-Windows.${ext}"
				)
			done
		else
			rm -f /var/lib/clamav/Atomicorp-Windows.* >/dev/null 2>&1
		fi
		if [[ ${FRESHCLAM_ANDROID_SIGS} == "yes" ]]; then
			for ext in $EXTENSIONS; do
				files_to_add+=(
					"Atomicorp-Android.${ext}"
				)
			done
		else
			rm -f /var/lib/clamav/Atomicorp-Android.* >/dev/null 2>&1
		fi
		if [[ ${FRESHCLAM_OSX_SIGS} == "yes" ]]; then
			for ext in $EXTENSIONS; do
				files_to_add+=(
					"Atomicorp-OSX.${ext}"
				)
			done
		else
			rm -f /var/lib/clamav/Atomicorp-OSX.* >/dev/null 2>&1
		fi

		for file in "${files_to_add[@]}"; do
			echo "DatabaseCustomURL https://rule-updates.atomicorp.com/channels/rules/anti-malware/${file}" >> $FRESHCLAM_CONF
		done


		# Paths to service files
		if [ -f /usr/lib/systemd/system/clamav-daemon.service ]; then
			service_path1="/usr/lib/systemd/system/clamav-daemon.service"
		else
			service_path1="/usr/lib/systemd/system/clamd@.service"
		fi
		service_path2="/lib/systemd/system/clamav-daemon.service"

		if [[ -f "$service_path1" ]]; then
			# Check if the ConditionPath statement exists and comment it out
    		if grep -q "ConditionPathExistsGlob=/var/lib/clamav/" "$service_path1"; then
        		sed -i 's/ConditionPathExistsGlob=\/var\/lib\/clamav\//#ConditionPathExistsGlob=\/var\/lib\/clamav\//g' "$service_path1"
    			systemctl daemon-reload
    		fi
		elif [[ -f "$service_path2" ]]; then

    		# Check if the ConditionPath statement exists and comment it out
    		if grep -q "ConditionPathExistsGlob=/var/lib/clamav/" "$service_path2"; then
        		sed -i 's/ConditionPathExistsGlob=\/var\/lib\/clamav\//#ConditionPathExistsGlob=\/var\/lib\/clamav\//g' "$service_path2"
    			systemctl daemon-reload
    		fi
		fi

		if [[ -e /var/lib/clamav/daily.cvd || /var/lib/clamav/main.cvd ]]; then
			FRESHCLAM_RELOAD=1
		fi


	fi

	if [[ $DEBUG -ge 1 ]]; then
		log_event "DEBUG1: freshclam settings updated: END"
	fi

}

freshclam_update() {
	if [[ $DEBUG -ge 1 ]]; then
		log_event "DEBUG1: freshclam update: START"
	fi

    if [[ "$PKG" == "rpm" ]]; then
       	if ! is_docker ; then
			systemctl stop freshclam
		fi
    elif [[ "$PKG" == "deb" ]]; then
       	if ! is_docker ; then
			systemctl stop clamav-freshclam
		fi
  	fi

	if [[ ${FRESHCLAM_RELOAD} == 1 ]]; then
		/var/ossec/modules/clamav/freshclam 
		RET=$?
	fi

	if [[ $DEBUG -ge 1 ]]; then
		log_event "DEBUG1: freshclam update: ($RET) END"
	fi
}

# create a function to quarantine files
quarantine_settings() {
	if [[ $DEBUG -ge 1 ]]; then
		log_event "DEBUG1: quarantine_settings:: START"
		log_event "DEBUG1: quarantine_settings:: QUARANTINE (${QUARANTINE})"
	fi

	if [[ ${QUARANTINE} == 1 ]]; then
		if [[ ${QUARANTINE_PATH} == "" ]]; then
			QUARANTINE_PATH="/var/awp/data/suspicious"
		fi

		# write to the file /etc/default/clamav-daemon
		if [[ ! -d /etc/default/ ]]; then
			mkdir /etc/default/
		fi
		echo "OPTIONS=\"--move=${QUARANTINE_PATH}\"" > /etc/default/clamav-daemon

	else
		if [[ -f /etc/default/clamav-daemon ]]; then
			sed -i 's/OPTIONS=.*/OPTIONS=""/g' /etc/default/clamav-daemon
		fi
	fi

	if [[ $REALTIME -eq 1 ]]; then

		if [ -f /usr/lib/systemd/system/clamav-clamonacc.service ]; then
			if [ -f /etc/systemd/system/clamonacc.service ]; then
				rm -f /etc/systemd/system/clamonacc.service
			fi
			systemctl enable clamav-clamonacc.service
			if [[ $DEBUG -ge 1 ]]; then
				log_event "DEBUG1: quarantine_settings:: enable clamonacc.service:($?) "
			fi
		else 
			cp /var/ossec/modules/clamav/template/clamonacc.service /etc/systemd/system/clamonacc.service
			systemctl daemon-reload
			systemctl enable clamonacc.service
			if [[ $DEBUG -ge 1 ]]; then
				log_event "DEBUG1: quarantine_settings:: enable clamonacc.service:($?) "
			fi
		fi
		if [[ $DEBUG -ge 1 ]]; then
			log_event "DEBUG1: quarantine_settings:: enable clamonacc.service:($?) "
		fi
	else
		if [ -f /usr/lib/systemd/system/clamav-clamonacc.service ]; then
			systemctl disable clamav-clamonacc
			systemctl stop clamav-clamonacc
		else
			systemctl disable clamonacc.service
			systemctl stop clamonacc.service
			rm -f /etc/systemd/system/clamonacc.service
			systemctl daemon-reload
		fi
	fi

	if [[ $DEBUG -ge 1 ]]; then
		log_event "DEBUG1: quarantine_settings:: END"
	fi
}

#######################
# Main
#######################
if [[ $DEBUG -ge 1 ]]; then
	log_event "DEBUG1: $0: START"
fi

dist_detection
clamd_version
clamd_settings

if [ -f /var/ossec/etc/shared/malware-detection ]; then
    CONFIG_FILE="/var/ossec/etc/shared/malware-detection"
elif [ -f /var/ossec/etc/shared/${ID}.malware-detection ]; then
    CONFIG_FILE="/var/ossec/etc/shared/${ID}.malware-detection"
else
    echo "Neither ${ID}.malware-detection nor malware-detection found."
    exit 1
fi
if [[ $DEBUG -ge 1 ]]; then
	log_event "DEBUG1: $0: config file is ($CONFIG_FILE) and clamd.conf is ($CLAM_CONF)"
fi

if [ -f "$CONFIG_FILE" ] && [ -f "$CLAM_CONF" ]; then

	if [[ $DEBUG -ge 1 ]]; then
		log_event "DEBUG1: Updating clamav settings: START"
	fi

   	sed -i "s/^User.*/User root/g" $CLAM_CONF
   	sed -i "s/^LocalSocketGroup.*//g" $CLAM_CONF
   	sed -i "s/^LocalSocketMode.*//g" $CLAM_CONF
   	sed -i "s/^OnAccessExcludeRootUID.*//g" $CLAM_CONF
   	sed -i "s/^OnAccessExcludeUID.*//g" $CLAM_CONF
   	sed -i "s/^OnAccessIncludePath.*//g" $CLAM_CONF
   	sed -i "s/^OnAccessExcludePath.*//g" $CLAM_CONF
   	sed -i "s/^OnAccessPrevention.*//g" $CLAM_CONF

	for line in $(cat $CONFIG_FILE ); do
		if [[ $DEBUG -ge 2 ]]; then
			log_event "DEBUG2: input is: ($line)"
		fi
		if  [[ ${line} == uid* ]]; then
			NAME=$(echo $line |awk -F, '{ print $2}')
			if id -u ${NAME} >/dev/null 2>&1 ; then
				CLAM_UID=$(id -u ${NAME})
       				if [[ ${CLAM_UID} -eq 0 ]]; then
					echo "OnAccessExcludeRootUID yes"  >> $CLAM_CONF
           			else
					echo "OnAccessExcludeUID ${CLAM_UID}" >> $CLAM_CONF
           			fi
			fi
		elif [[ ${line} == include* ]]; then
       			CLAM_INCLUDE=$(echo $line |awk -F, '{ print $2}')
			if [ -d $CLAM_INCLUDE ]; then
       				echo "OnAccessIncludePath  $CLAM_INCLUDE" >> $CLAM_CONF
			fi
		        INCLUDE_PATHS=$((INCLUDE_PATHS + 1))
		elif [[ ${line} == exclude* ]]; then
       			CLAM_EXCLUDE=$(echo $line |awk -F, '{ print $2}')
       			echo "OnAccessExcludePath $CLAM_EXCLUDE" >> $CLAM_CONF
		elif [[ ${line} == setting,* ]]; then

			CLAM_SETTING=$(echo $line |awk -F, '{ print $2}')
			CLAM_TOKEN=$(echo $line |awk -F, '{ print $3}')

			# Freshclam settings
			if [[ ${CLAM_SETTING} == freshclam_* ]]; then
				# if CLAM_SETTING is freshclam_feed_type, and CLAM_TOKEN is atomic then we will process values for linux_sigs, windows_sigs, and android_sigs
				if [[ ${CLAM_SETTING} == "freshclam_feed_type" ]] && [[ ${CLAM_TOKEN} == "atomic" ]]; then
					FRESHCLAM_FEED_TYPE="atomic"
				elif [[ ${CLAM_SETTING} == "freshclam_feed_type" ]] && [[ ${CLAM_TOKEN} == "official" ]]; then
					FRESHCLAM_FEED_TYPE="official"
				fi

				if [[ ${CLAM_SETTING} == "freshclam_linux_sigs" ]] && [[ ${CLAM_TOKEN} == "true" ]]; then
					FRESHCLAM_LINUX_SIGS="yes"
				fi
				if [[ ${CLAM_SETTING} == "freshclam_windows_sigs" ]] && [[ ${CLAM_TOKEN} == "true" ]]; then
					FRESHCLAM_WINDOWS_SIGS="yes"
				fi
				if [[ ${CLAM_SETTING} == "freshclam_android_sigs" ]] && [[ ${CLAM_TOKEN} == "true" ]]; then
					FRESHCLAM_ANDROID_SIGS="yes"
				fi
				if [[ ${CLAM_SETTING} == "freshclam_osx_sigs" ]] && [[ ${CLAM_TOKEN} == "true" ]]; then
					FRESHCLAM_OSX_SIGS="yes"
				fi
			elif [[ ${CLAM_SETTING} == "quarantine" ]]; then
				if [[ $CLAM_TOKEN == "true" ]]; then
					QUARANTINE=1
				fi
			elif [[ ${CLAM_SETTING} == "quarantine_path" ]]; then
				QUARANTINE_PATH=${CLAM_TOKEN}
			elif [[ ${CLAM_SETTING} == "enable" ]]; then
				if [[ $DEBUG -ge 1 ]]; then
					log_event "DEBUG1: CLAM_SETTING is ${CLAM_TOKEN}"
				fi
				if [[ $CLAM_TOKEN == "true" ]]; then
					ENABLE=1
				else
					ENABLE=0
				fi
			else 
			    if grep -q "^${CLAM_SETTING}" $CLAM_CONF; then
			    	sed -i "s/^${CLAM_SETTING}.*/${CLAM_SETTING} ${CLAM_TOKEN}/g" $CLAM_CONF
			    else
			    	echo "${CLAM_SETTING} ${CLAM_TOKEN}" >> $CLAM_CONF
			    fi

			    if [[ ${CLAM_SETTING} == "OnAccessPrevention" ]] && [[ ${CLAM_TOKEN} == "true" ]]; then
				REALTIME=1
			    fi

			fi
					
		fi

	done

	sed -i '/^$/d' $CLAM_CONF
	freshclam_settings
	freshclam_update
	quarantine_settings
	clamd_restart
	if [[ $DEBUG -ge 1 ]]; then
		log_event "DEBUG1: clamav settings updated: END"
	fi
else
	if [[ $DEBUG -ge 1 ]]; then
		log_event "DEBUG1: CLAM config $CONFIG_FILE and $CLAM_CONF not detected"
	fi

	
fi

if [[ $DEBUG -ge 1 ]]; then
	log_event "DEBUG1: $0: END"
fi
