#!/usr/bin/bash
# Atomicorp Malware Scanner Active Response Script
#
# Copyright Atomicorp 2024
# All Rights reserved

VERSION=1.2.4
HUB_IP=$(grep -o -P '(?<=<address>).*(?=</address>)' /var/ossec/etc/ossec.conf)


if [ -f /etc/default/clamav-daemon ]; then
	source /etc/default/clamav-daemon
fi



# Planned
if [ -f /var/ossec/etc/malware-scanner.conf ]; then
	source /var/ossec/etc/malware-scanner.conf
else
	CLAMAV_SERVER=daemon
fi


# Detect Fedora/ RedHat 9 style clamav 
if [ -f /etc/freshclam.conf ] && [ -f /etc/clamd.d/scan.conf ]; then
    DISTRO=rh9like
fi

###########################
# Logging
###########################
LOG_FILE="/var/ossec/logs/active-responses.log"

###########################
#functions
###########################
function log_event () {
	MSG=$1
	echo "`date "+%b %d %H:%M:%S"` `hostname` atomicorp-api: ${MSG}" >> ${LOG_FILE}
}



function update_clamav_signatures() {
	log_event "updating malware rules" 
    if [[ $OSTYPE == "aix"* ]]; then
        /opt/freeware/bin/freshclam
    else
        if [ ! -d /var/lib/clamav ]; then
            mkdir /var/lib/clamav
        fi
        /var/ossec/modules/clamav/freshclam
    fi
	# if this fails, log the failure, else log the success
	if [ $? -ne 0 ]; then
		log_event "freshclam update failed"
	else
       	log_event "completed malware rule update" 
	fi
}

function init_clamav_conf() {
	if [[ $OSTYPE == "aix"* ]]; then
		cp -f /var/ossec/modules/clamav/template/clamd.conf.template /opt/freeware/etc/clamd.conf
		cp -f /var/ossec/modules/clamav/template/freshclam.conf.template /opt/freeware/etc/freshclam.conf
		if [ ! -d /var/log ]; then
			mkdir -p /var/log
		fi

		if [ ! -f /var/log/messages ]; then
			touch /var/log/messages
		fi
	else
		/var/ossec/modules/clamav/clam-module.sh
	fi
}


function install_clamav(){
    if [[ $OSTYPE == "aix"* ]]; then
		echo
	elif [[ $DISTRO == "rh9like" ]]; then
		if ! rpm -q clamd ; then
			yum install -y clamd
		fi
		if ! rpm -q clamav ; then
			yum install -y clamav
		fi
	else
		if ! rpm -q clamd ; then
			yum -y install clamd
		fi
	fi
}

# add a function to configure remote clamav server  
function init_clamav_remote_conf(){
	if [[ $OSTYPE == "aix"* ]]; then

		# Configure stunnel.conf
		cp -f /var/ossec/modules/clamav/template/stunnel.conf /opt/freeware/etc/stunnel/stunnel.conf
		/opt/freeware/bin/sed -i "s/CLAMAV_SERVER/${HUB_IP}/g" /opt/freeware/etc/stunnel/stunnel.conf

		# Configure clamd.conf
		cp -f /var/ossec/modules/clamav/template/clamd.remote.conf /opt/freeware/etc/clamd.conf

		# Create /var/log/clamav if it doesn't exist
		if [ ! -d /var/log/clamav ]; then
			mkdir -p /var/log/clamav
		fi

		# add stunnel-aix.sh to startup
		cp -f /var/ossec/modules/clamav/template/stunnel-aix.sh /etc/rc.d/rc2.d/S99stunnel.sh

		# start stunnel
		/opt/freeware/bin/stunnel /opt/freeware/etc/stunnel/stunnel.conf

	fi
}

is_stdin_available() {
    if [ -t 0 ]; then
        return 1 # stdin is empty (terminal)
    else
        return 0 # stdin has data
    fi
}



###########################
# Main
###########################
if [ "$1" == "add" ] ||  [ "$1" == "delete" ]; then
        ACTION=$1
        shift
        shift
        ARGS="$@"
else
        read -r stdin_data
        ARGS=$(echo "$stdin_data" | jq -r .parameters.alert.data.srcip)
fi




if [[ "$ARGS" == "update"* ]]; then
	update_clamav_signatures

elif [[ "$ARGS" == "init-local"* ]]; then

	log_event "malware scanner initializing local config"

	if [[ $OSTYPE == "aix"* ]]; then
		init_clamav_conf
		update_clamav_signatures
	elif [[ $DISTRO == "rh9like" ]]; then
	    install_clamav
		init_clamav_conf
		update_clamav_signatures
		systemctl restart clamd@scan
		RETVAL=$?
		if [ $RETVAL -eq 0 ]; then
			log_event "malware scanner initialization completed successfully"
		else
			log_event "malware scanner initialization failed (001)"
		fi
	else
		install_clamav
		init_clamav_conf
		update_clamav_signatures
		systemctl restart clamav-daemon
		RETVAL=$?
		if [ $RETVAL -eq 0 ]; then
			log_event "malware scanner initialization completed successfully"
		else
			log_event "malware scanner initialization failed (002)"
		fi
		
	fi
# Planned
elif [[ "$ARGS" == "init-remote"* ]]; then
	log_event "malware scanner initializing remote config"
	if [[ $OSTYPE == "aix"* ]]; then
		init_clamav_remote_conf
	fi
else 
    log_event "running malware scan on ${ARGS}" 
	TARGET=${ARGS%% scanid:*}
	SCANID=${ARGS##*scanid:}

	if [[ $OSTYPE == "aix"* ]]; then
		CLAM_PATH=/opt/freeware/bin 
		if [ -f /opt/freeware/etc/clamd.conf ]; then
			CLAM_CONF=/opt/freeware/etc/clamd.conf
		elif [ -f /etc/clamd-remote.conf ]; then
			CLAM_CONF=/etc/clamd-remote.conf
		else
			CLAMAV_SERVER=local
		fi
	
	elif [[ $DISTRO == "rh9like" ]]; then
		CLAM_PATH=/usr/bin 
		CLAM_CONF=/etc/clamd.d/scan.conf

	else
		CLAM_PATH=/usr/bin
        if [ -f /etc/clamd.conf ]; then
            CLAM_CONF=/etc/clamd.conf
        elif [ -f /etc/clamav/clamd.conf ]; then
            CLAM_CONF=/etc/clamav/clamd.conf
        elif [ -f /etc/clamd.d/scan.conf ]; then
            CLAM_CONF=/etc/clamd.d/scan.conf
        fi
		
	fi


	# Scan Type
	if [[ $CLAMAV_SERVER == "daemon" ]]; then
		# remote scanner
		RESULT=$($CLAM_PATH/clamdscan -c $CLAM_CONF --stream -i -m  --no-summary $OPTIONS ${TARGET})
	else
		# local
		RESULT=$($CLAM_PATH/clamscan -i --no-summary $OPTIONS ${TARGET})
	fi

	IFS=$'\n'
	for line in $RESULT; do
		if [[ $OSTYPE == "aix"* ]]; then
			DATE=$(date "+%b %d %H:%M:%S")
			echo "${DATE} `hostname` clamd: $line scanid:${SCANID}" >> /var/log/messages
		else

      		logger -t clamd "$line scanid:${SCANID}"
		fi

	done
	unset IFS
    log_event "completed malware scan on ${TARGET}" 

fi

